﻿using UnityEngine;
using System.Collections;

public class EnemyController : MonoBehaviour
{
	public float disappearDistance = 15.0f;
	public float disappearTime = 1.0f;
	public float moveDuration = 0.1f;

	public float changeDirectionIntervalMin = 1.0f;
	public float changeDirectionInternalMax = 3.0f;
	private float timeLeftUntilChange;
	private float currentDX;
	private float currentDY;

	private CharacterMove _move;

	void Start ()
	{
		_move = GetComponent<CharacterMove>();
	}
	
	void Update ()
	{
		// die if far away from player or player does not exist
		var player = PlayerController.instance;
		if (!player)
		{
			StartCoroutine (Die ());
			return;
		}

		if ((transform.position - player.transform.position).sqrMagnitude > disappearDistance * disappearDistance)
		{
			StartCoroutine (Die ());
			return;
		}

		// movement logic
		if (timeLeftUntilChange <= 0.0f)
		{
			timeLeftUntilChange = Random.Range (changeDirectionIntervalMin, changeDirectionInternalMax);
			var dirIndex = Random.Range (0,6);
			switch(dirIndex) {
			case 0: currentDX = -1; currentDY =  0; break;
			case 1: currentDX =  0; currentDY = -1; break;
			case 2: currentDX =  1; currentDY =  0; break;
			case 3: currentDX =  0; currentDY =  1; break;
			default: currentDX = 0; currentDY = 0; break;
			}
		}
		timeLeftUntilChange -= Time.deltaTime;

		if (currentDX != 0 || currentDY != 0)
		{
			if (!_move.Move(currentDX, currentDY, moveDuration))
			{
				timeLeftUntilChange = 0;
			}
		}
	}

	public void DoDamage()
	{
		PlayerController.instance.AddScore(1);
		StartCoroutine (Die ());
	}

	IEnumerator Die()
	{
		var sr = GetComponent<SpriteRenderer>();
		var col = sr.color;
		while (col.a > 0.0f)
		{
			col.a -= Time.deltaTime / disappearTime;
			sr.color = col;
			yield return 0;
		}
		Object.Destroy (gameObject);
	}
}
